from pyamg.testing import *

from numpy import matrix, array, diag, zeros, sqrt, abs, ravel, ones, arange, eye
from scipy import rand, linalg, real, imag, mat, diag, isscalar, ones, hstack
from scipy.sparse import csr_matrix, isspmatrix, bsr_matrix, isspmatrix_bsr, spdiags

import pyamg
from pyamg.util.utils import *

class TestUtils(TestCase):
    def test_diag_sparse(self):
        #check sparse -> array
        A = matrix([[-4]])
        assert_equal(diag_sparse(csr_matrix(A)),[-4])

        A = matrix([[1,0,-5],[-2,5,0]])
        assert_equal(diag_sparse(csr_matrix(A)),[1,5])

        A = matrix([[0,1],[0,-5]])
        assert_equal(diag_sparse(csr_matrix(A)),[0,-5])

        A = matrix([[1.3,-4.7,0],[-2.23,5.5,0],[9,0,-2]])
        assert_equal(diag_sparse(csr_matrix(A)),[1.3,5.5,-2])

        #check array -> sparse
        A = matrix([[-4]])
        assert_equal(diag_sparse(array([-4])).todense(),csr_matrix(A).todense())

        A = matrix([[1,0],[0,5]])
        assert_equal(diag_sparse(array([1,5])).todense(),csr_matrix(A).todense())

        A = matrix([[0,0],[0,-5]])
        assert_equal(diag_sparse(array([0,-5])).todense(),csr_matrix(A).todense())

        A = matrix([[1.3,0,0],[0,5.5,0],[0,0,-2]])
        assert_equal(diag_sparse(array([1.3,5.5,-2])).todense(),csr_matrix(A).todense())


    def test_symmetric_rescaling(self):
        cases = []
        cases.append( diag_sparse(array([1,2,3,4])) )
        cases.append( diag_sparse(array([1,0,3,4])) )

        A = array([ [ 5.5,  3.5,  4.8],
                    [ 2. ,  9.9,  0.5],
                    [ 6.5,  2.6,  5.7]])
        A = csr_matrix( A )
        cases.append(A)
        P = diag_sparse([1,0,1])
        cases.append( P*A*P )
        P = diag_sparse([0,1,0])
        cases.append( P*A*P )
        P = diag_sparse([1,-1,1])
        cases.append( P*A*P )

        for A in cases:
            D_sqrt,D_sqrt_inv,DAD = symmetric_rescaling(A)

            assert_almost_equal( diag_sparse(A) > 0, diag_sparse(DAD) )
            assert_almost_equal( diag_sparse(DAD), D_sqrt*D_sqrt_inv )

            D_sqrt,D_sqrt_inv = diag_sparse(D_sqrt),diag_sparse(D_sqrt_inv)
            assert_almost_equal((D_sqrt_inv*A*D_sqrt_inv).todense(), DAD.todense())
    
    def test_symmetric_rescaling_sa(self):
        cases = []
        # case 1
        e = ones((5,1)).ravel()
        data = [ -1*e, 2*e, -1*e ]
        A = spdiags(data,[-1,0,1],5,5).tocsr()
        B = e.copy().reshape(-1,1)
        DAD_answer = array([[ 1. ,-0.5, 0. , 0. , 0. ],
                            [-0.5, 1. ,-0.5, 0. , 0. ],
                            [ 0. ,-0.5, 1. ,-0.5, 0. ],
                            [ 0. , 0. ,-0.5, 1. ,-0.5],
                            [ 0. , 0. , 0. ,-0.5, 1. ]])
        DB_answer = sqrt(2*e.reshape(-1,1))
        #            matrix   B    BH   expected matrix  expected B  expected BH 
        cases.append( (A,     B,  None,   DAD_answer,      DB_answer,   None) )

        # case 2
        A2 = A.copy()
        A2.symmetry = 'nonsymmetric'
        cases.append( (A2, B.copy(), B.copy(), DAD_answer, DB_answer, DB_answer) )
        
        # case 3
        A3 = A.copy()
        A3.symmetry = 'hermitian'
        cases.append( (A3, B.copy(), None, DAD_answer, DB_answer, None) )

        # case 4
        B4 = hstack( (B.copy(), 2*B.copy()) )
        DB4_answer = sqrt(2)*B4
        A4 = A.copy()
        A4.symmetry = 'nonsymmetric'
        cases.append( (A4, B4, B4.copy(), DAD_answer, DB4_answer, DB4_answer) )
        

        for case in cases:
            [A,B,BH,DAD_answer,DB_answer,DBH_answer] = case
            
            [DAD, DB, DBH] = symmetric_rescaling_sa(A,B,BH=BH)
            assert_array_almost_equal(DAD.todense(), DAD_answer)
            assert_array_almost_equal(DB, DB_answer)
            if DBH_answer is not None:
                assert_array_almost_equal(DBH, DBH_answer)


    def test_profile_solver(self):
        from scipy.sparse.linalg import cg
        from pyamg.gallery import poisson
        from pyamg.aggregation import smoothed_aggregation_solver

        A = poisson((100,100), format='csr')
        ml = smoothed_aggregation_solver(A)

        opts = []
        opts.append( {} )
        opts.append( {'accel' : cg } )
        opts.append( {'accel' : cg, 'tol' : 1e-10 } )

        for kwargs in opts:
            residuals = profile_solver(ml, **kwargs)

    def test_get_block_diag(self):
        from scipy import arange, ravel, array
        from scipy.sparse import csr_matrix
        A = csr_matrix(arange(1,37, dtype=float).reshape(6,6))
        block_diag = get_block_diag(A, blocksize=1, inv_flag=False)
        assert_array_almost_equal(ravel(block_diag), A.diagonal())

        block_diag = get_block_diag(A, blocksize=2, inv_flag=False)
        answer = array([[[  1.,   2.],
                         [  7.,   8.]],
                        [[ 15.,  16.],
                         [ 21.,  22.]],
                        [[ 29.,  30.],
                         [ 35.,  36.]]])
        assert_array_almost_equal(ravel(block_diag), ravel(answer))

        block_diag_inv = get_block_diag(A, blocksize=2, inv_flag=True)
        answer = array([[[-1.33333333,  0.33333333],
                         [ 1.16666667, -0.16666667]],
                        [[-3.66666667,  2.66666667],
                         [ 3.5       , -2.5       ]],
                        [[-6.        ,  5.        ],
                         [ 5.83333333, -4.83333333]]])
        assert_array_almost_equal(ravel(block_diag_inv), ravel(answer), decimal=3)

        # try with singular (1,1) block, a zero (2,2) block and a zero (0,2) block
        A = bsr_matrix(array([ 1.,  2.,  3.,  4.,  0.,  0.,
                               5.,  6.,  7.,  8.,  0.,  0.,
                               9., 10., 11., 11., 12., 13.,
                              14., 15., 16., 16., 18., 19.,
                              20., 21., 22., 23.,  0.,  0.,
                              26., 27., 28., 29.,  0.,  0.,]).reshape(6,6), blocksize=(3,3))
        block_diag_inv = get_block_diag(A, blocksize=2, inv_flag=True)
        answer = array([[[-1.5       ,  0.5       ],
                         [ 1.25      , -0.25      ]],
                        [[ 0.01458886,  0.02122016],
                         [ 0.01458886,  0.02122016]],
                        [[ 0.        ,  0.        ],
                         [ 0.        ,  0.        ]]])
        assert_array_almost_equal(ravel(block_diag_inv), ravel(answer), decimal=3)

        # try with different types of zero blocks
        A = bsr_matrix(array([ 0.,  0.,  3.,  4.,  0.,  0.,
                               0.,  0.,  7.,  8.,  0.,  0.,
                               0.,  0.,  0.,  0.,  0.,  0.,
                               0.,  0.,  0.,  0.,  0.,  0.,
                               0.,  0.,  0.,  0., 22., 23.,
                               0.,  0.,  0.,  0., 28., 29.,]).reshape(6,6), blocksize=(2,2))
        block_diag_inv = get_block_diag(A, blocksize=2, inv_flag=False)
        answer = array([[[ 0.        ,  0.        ],
                         [ 0.        ,  0.        ]],
                        [[ 0.        ,  0.        ],
                         [ 0.        ,  0.        ]],
                        [[22.        , 23.        ],
                         [28.        , 29.        ]]])
        assert_array_almost_equal(ravel(block_diag_inv), ravel(answer), decimal=3)


    def test_relaxation_as_linear_operator(self):
        As = []
        bs =[]
        xs = []
        methods = ['gauss_seidel', 'jacobi', 'block_gauss_seidel', 'block_jacobi']
        params = [{}, {'iterations' : 2}]
        As.append(pyamg.gallery.poisson( (10,10), format='csr'))
        As.append(1.0j*pyamg.gallery.poisson( (10,10), format='csr'))
        As.append(1.0j*pyamg.gallery.elasticity.linear_elasticity( (20,20) )[0] )
        As.append(pyamg.gallery.elasticity.linear_elasticity( (20,20) )[0] )
        for A in As:
            if A.dtype == 'complex':
                xs.append(rand(A.shape[0],1)+1.0j*rand(A.shape[0],1))
                bs.append(rand(A.shape[0],1)+1.0j*rand(A.shape[0],1))
            else:
                bs.append(rand(A.shape[0],1))
                xs.append(rand(A.shape[0],1))

        for method in methods:
            for kwargs in params:    
                for (A,x,b) in zip(As,xs,bs):
                    kwargs_linop = dict(kwargs)
                    ##
                    # run relaxation as a linear operator
                    if kwargs_linop == dict({}):
                        relax = relaxation_as_linear_operator(method, A, b)
                    else:
                        relax = relaxation_as_linear_operator((method,kwargs_linop), A, b)
                    x_linop = relax*x
                    
                    ##
                    # manually run the relaxation routine
                    relax2 = eval('pyamg.relaxation.' + method)
                    x_gold = x.copy()
                    blockflag = False
                    kwargs_gold = dict(kwargs)
                    # deal with block matrices
                    if method.startswith('block') and isspmatrix_bsr(A):
                        blockflag = True
                        kwargs_gold['blocksize'] = A.blocksize[0]
                    # deal with omega and jacobi
                    # --> note that we assume the default setup for jacobi uses omega = 1/rho
                    if method.endswith('jacobi'):
                        if blockflag:
                            kwargs_gold['omega'] = 1.0/A.rho_block_D_inv
                        else:
                            kwargs_gold['omega'] = 1.0/A.rho_D_inv

                    relax2(A,x_gold,b,**kwargs_gold)
                    
                    assert_array_almost_equal(x_linop, x_gold)

    def test_filter_operator(self):
        ##
        # Basic tests of dimension 1 and 2 problems
        # 1x1
        A = csr_matrix(array([[1.2]]))
        C = csr_matrix(array([[1.]]))
        B = array([[0.5]])
        Bf= array([[1.5]])
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = matrix([[3.0]])
        assert_array_almost_equal(A_known, A_filter)
        # 1x1, but with no entries in C
        C = csr_matrix(array([[0.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0.0]])
        assert_array_almost_equal(A_known, A_filter)
        # 1x1, but with no entries in A
        A = csr_matrix(array([[0.]]))
        C = csr_matrix(array([[1.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[3.0]])
        assert_array_almost_equal(A_known, A_filter)


        # 1x2
        A = csr_matrix(array([[1.2, 1.]]))
        C = csr_matrix(array([[1.,  1.]]))
        B = array([[0.5], [0.5]])
        Bf= array([[1.5]])
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = matrix([[ 1.6,  1.4]])
        assert_array_almost_equal(A_known, A_filter)
        # 1x2, but sparser
        C = csr_matrix(array([[0.,  1.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0., 3.]])
        assert_array_almost_equal(A_known, A_filter)
        # 1x2, but with no entries
        C = csr_matrix(array([[0.,  0.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0., 0.]])
        assert_array_almost_equal(A_known, A_filter)

        # 2x1
        A = csr_matrix(array([[1.2], [1.]]))
        C = csr_matrix(array([[1.],  [1.]]))
        B = array([[0.5]])
        Bf= array([[1.5], [0.4]])
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = matrix([[ 3.],  [0.8]])
        assert_array_almost_equal(A_known, A_filter)
        # 2x1, but sparser
        C = csr_matrix(array([[0.],  [1.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0.], [.8]])
        assert_array_almost_equal(A_known, A_filter)
        # 2x1, but with no entries
        C = csr_matrix(array([[0.],  [0.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0.], [0.]])
        assert_array_almost_equal(A_known, A_filter)

        # 2x2
        A = csr_matrix(array([[1.2, 1.1], [1., 0.5]]))
        C = csr_matrix(array([[1.2, 1.1], [1., 0.]]))
        B = array([[0.5, 1.0],[0.5, 1.1]])
        Bf = array([[0.5,1.0],[0.5,1.1]])
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[1., 0.], [1.08, 0.]])
        assert_array_almost_equal(A_known, A_filter)
        # 1x2, but sparser
        C = csr_matrix(array([[0., 0.], [1., 0.]]))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[0., 0.], [1.08, 0.]])
        assert_array_almost_equal(A_known, A_filter)
        # Try block structure
        A = A.tobsr((2,2))
        C = C.tobsr((2,2))
        A_filter = filter_operator(A, C, B, Bf).todense()
        A_known = array([[1., 0.], [0., 1.]])
        assert_array_almost_equal(A_known, A_filter)
        
        ##
        # Basic tests, with easy to compute answers
        # test one, the constant
        A = array([ [1.,1,1],[1,1,1],[0,1,0],[0,1,0],[0,0,1],[0,0,1]])
        C = array([ [1.,1,0],[1,1,0],[0,1,0],[0,1,0],[0,0,1],[0,0,1]])
        B = ones((3,1))
        Bf = ones((6,1))
        A_filter = filter_operator(csr_matrix(A), csr_matrix(C), B, Bf).todense()
        A_known = matrix([[ 0.5,  0.5,  0. ],
                          [ 0.5,  0.5,  0. ],
                          [ 0. ,  1. ,  0. ],
                          [ 0. ,  1. ,  0. ],
                          [ 0. ,  0. ,  1. ],
                          [ 0. ,  0. ,  1. ]])
        assert_array_almost_equal(A_known, A_filter)
        # test two, the constant and linears
        B = hstack( (B, arange(B.shape[0]).reshape(-1,1)) )
        Bf = hstack( (Bf, arange(Bf.shape[0]).reshape(-1,1)) )
        A_filter = filter_operator(csr_matrix(A), csr_matrix(C), B, Bf).todense()
        A_known = matrix([[ 1. ,  0. ,  0. ],
                          [ 0. ,  1. ,  0. ],
                          [ 0. ,  1.5,  0. ],
                          [ 0. ,  2. ,  0. ],
                          [ 0. ,  0. ,  1.8],
                          [ 0. ,  0. ,  2.2]])
        assert_array_almost_equal(A_known, A_filter)
        
        ##
        # Run two tests based on the Laplacian
        # first test, constants
        from pyamg.gallery import poisson
        A = poisson((10,10), format='csr')
        C = A.copy()
        C.data[arange(0, C.nnz, 5) ] = 0.0
        C.eliminate_zeros()
        B = ones((A.shape[0],1))
        Bf = ones((A.shape[0],1))
        A_filter = filter_operator(A, C, B, Bf)
        assert_array_almost_equal(A_filter*B, Bf)
        # second test, constants and linears
        B = hstack( (B, arange(B.shape[0]).reshape(-1,1)) )
        Bf = hstack( (Bf, arange(Bf.shape[0]).reshape(-1,1)) )
        A_filter = filter_operator(A, C, B, Bf)
        assert_array_almost_equal(A_filter*B, Bf)

    def test_scale_T(self):
        from scipy.sparse import csr_matrix, bsr_matrix
        from scipy import matrix

        ##
        # Trivially sized tests
        # 1x1
        T   = matrix([[ 1.1 ]] )
        P_I = matrix([[ 1.0 ]] )
        I_F = matrix([[ 0.0 ]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 1.0 ]] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 1.1 ]] )
        P_I = matrix([[ 0.0 ]] )
        I_F = matrix([[ 1.0 ]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 1.1 ]] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 0.0 ]] )
        P_I = matrix([[ 0.0 ]] )
        I_F = matrix([[ 1.0 ]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 0. ]] )
        assert_array_almost_equal(T_answer, T_scaled)

        # 2x1
        T   = matrix([[ 1.5 ], [1.2] ] )
        P_I = matrix([[ 1.  ], [0. ] ] )
        I_F = matrix([[ 0., 0. ], [0., 1.]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 1. ], [0.8] ] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 0.  ], [1.2] ] )
        P_I = matrix([[ 1.  ], [0. ] ] )
        I_F = matrix([[ 0., 0. ], [0., 1.]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 1. ], [0. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 0.  ], [0. ] ] )
        P_I = matrix([[ 1.  ], [0. ] ] )
        I_F = matrix([[ 0., 0. ], [0., 1.]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 1. ], [0. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 0.  ], [0. ] ] )
        P_I = matrix([[ 0.  ], [0. ] ] )
        I_F = matrix([[ 1., 0. ], [0., 1.]] )
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        T_answer = matrix([[ 0. ], [0. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)

        # 2x2
        T   = matrix([[ 2., 0. ], [1., 1. ] ] )
        P_I = matrix([[ 1., 0. ], [0., 1. ] ] )
        I_F = matrix([[ 0., 0. ], [0., 0. ]] )
        T_scaled = scale_T(bsr_matrix(T,  blocksize=(1,1)), 
                           bsr_matrix(P_I,blocksize=(1,1)), 
                           bsr_matrix(I_F,blocksize=(1,1))).todense()
        T_answer = matrix([[ 1., 0. ], [0., 1. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 2., 0. ], [1., 1. ] ] )
        P_I = matrix([[ 1., 0. ], [0., 1. ] ] )
        I_F = matrix([[ 0., 0. ], [0., 0. ]] )
        T_scaled = scale_T(bsr_matrix(T,  blocksize=(2,2)), 
                           bsr_matrix(P_I,blocksize=(2,2)), 
                           bsr_matrix(I_F,blocksize=(2,2))).todense()
        T_answer = matrix([[ 1., 0. ], [0., 1. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)
        ##
        T   = matrix([[ 2., 0. ], [1., 1. ] ] )
        P_I = matrix([[ 0., 0. ], [0., 0. ] ] )
        I_F = matrix([[ 1., 0. ], [0., 1. ]] )
        T_scaled = scale_T(bsr_matrix(T,  blocksize=(2,2)), 
                           bsr_matrix(P_I,blocksize=(2,2)), 
                           bsr_matrix(I_F,blocksize=(2,2))).todense()
        T_answer = matrix([[ 2., 0. ], [1., 1. ] ] )
        assert_array_almost_equal(T_answer, T_scaled)

        ##
        # Test for one CSR and one BSR example
        T = matrix([[ 1.0,  0.,   0. ],
                    [ 0.5,  0.,   0. ],
                    [ 0. ,  1.,   0. ],
                    [ 0. ,  0.5,  0. ],
                    [ 0. ,  0.,   1. ],
                    [ 0. ,  0.,   0.25 ]])
        P_I = matrix([[ 0.,  0.,   0. ],
                      [ 1.,  0.,   0. ],
                      [ 0.,  1.,   0. ],
                      [ 0.,  0.,   0. ],
                      [ 0.,  0.,   0. ],
                      [ 0.,  0.,   1. ]])
        I_F = matrix([[ 1.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  1.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.]])
        T_answer = matrix([[ 2. ,  0. ,  0. ],
                           [ 1. ,  0. ,  0. ],
                           [ 0. ,  1. ,  0. ],
                           [ 0. ,  0.5,  0. ],
                           [ 0. ,  0. ,  4. ],
                           [ 0. ,  0. ,  1. ]])
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        assert_array_almost_equal(T_answer, T_scaled)

        ##
        # BSR test
        T = matrix([[ 1.0, 1., 0.,  0. ],
                    [ 0.5, 1., 0.,  0. ],
                    [ 1. , 0., 0.,  0. ],
                    [ 0. , 1., 0.,  0. ],
                    [ 0. , 0., 2.,  1. ],
                    [ 0. , 0., 3.,  1. ],
                    [ 0. , 0., 4.,  1. ],
                    [ 0. , 0., 2.,  0. ]])
        P_I = matrix([[ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.],
                      [ 1.,  0.,  0.,   0.],
                      [ 0.,  1.,  0.,   0.],
                      [ 0.,  0.,  1.,   0.],
                      [ 0.,  0.,  0.,   1.],
                      [ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.]])
        I_F = matrix([[ 1.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  1.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.]])
        T_answer = matrix([[ 1. , 1.,  0.,  0. ],
                           [ 0.5, 1.,  0.,  0. ],
                           [ 1. , 0.,  0.,  0. ],
                           [ 0. , 1.,  0.,  0. ],
                           [ 0. , 0.,  1.,  0. ],
                           [ 0. , 0.,  0.,  1. ],
                           [ 0. , 0., -1.,  2. ],
                           [ 0. , 0., -2.,  2. ]]) 
        T = bsr_matrix(T, blocksize=(2,2))
        P_I = bsr_matrix(P_I, blocksize=(2,2))
        I_F = bsr_matrix(I_F, blocksize=(2,2))
        T_scaled = scale_T(T, P_I, I_F).todense()
        assert_array_almost_equal(T_answer, T_scaled)

        ##
        # BSR test
        T = matrix([[ 1.0, 1., 0.,  0. ],
                    [ 0.5, 1., 0.,  0. ],
                    [ 1. , 1., 0.,  0. ],
                    [ 1. , 1., 0.,  0. ],
                    [ 0. , 0., 2.,  1. ],
                    [ 0. , 0., 3.,  1. ],
                    [ 0. , 0., 4.,  1. ],
                    [ 0. , 0., 2.,  0. ]])
        P_I = matrix([[ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.],
                      [ 1.,  0.,  0.,   0.],
                      [ 0.,  1.,  0.,   0.],
                      [ 0.,  0.,  1.,   0.],
                      [ 0.,  0.,  0.,   1.],
                      [ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.]])
        I_F = matrix([[ 1.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  1.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.]])
        T_answer = matrix([[ 0.5  , 0.5  , 0.,  0. ],
                           [ 0.375, 0.375, 0.,  0. ],
                           [ 1.   , 0.   , 0.,  0. ],
                           [ 0.   , 1.   , 0.,  0. ],
                           [ 0.   , 0.   , 1.,  0. ],
                           [ 0.   , 0.   , 0.,  1. ],
                           [ 0.   , 0.   ,-1.,  2. ],
                           [ 0.   , 0.   ,-2.,  2. ]]) 
        Cpts = array([1,2])
        T = bsr_matrix(T, blocksize=(2,2))
        P_I = bsr_matrix(P_I, blocksize=(2,2))
        I_F = bsr_matrix(I_F, blocksize=(2,2))
        T_scaled = scale_T(T, P_I, I_F).todense()
        assert_array_almost_equal(T_answer, T_scaled)
    
    def test_get_Cpt_params(self):
        from pyamg.gallery import poisson
        from scipy.sparse import csr_matrix, bsr_matrix

        ##
        # Begin with trivially sized tests
        # 1x1
        A = csr_matrix(array([[1.2]]))
        Cpts = array([0])
        AggOp = csr_matrix(array([[1. ]]))
        T = AggOp.copy().tobsr()
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[ 1.]]), blocksize=(1,1))
        I_F = bsr_matrix( array([[ 0.]]), blocksize=(1,1))
        P_I = bsr_matrix( array([[ 1.]]), blocksize=(1,1) )
        assert_equal(array([0]),  params['Cpts'])
        assert_equal(array([ ]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        A = csr_matrix(array([[1.2]]))
        Cpts = array([ ])
        AggOp = csr_matrix(array([[1. ]]))
        T = AggOp.copy().tobsr()
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[ 0.]]), blocksize=(1,1))
        I_F = bsr_matrix( array([[ 1.]]), blocksize=(1,1))
        P_I = bsr_matrix( array([[ 0.]]), blocksize=(1,1) )
        assert_equal(array([ ]),  params['Cpts'])
        assert_equal(array([0]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        # 2x2
        A = csr_matrix(array([[1., 1.],[1., 1.]]))
        Cpts = array([0])
        AggOp = csr_matrix(array([[1.], [1.]]))
        T = AggOp.copy().tobsr()
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[1., 0.],[0., 0.]]), blocksize=(1,1))
        I_F = bsr_matrix( array([[0., 0.],[0., 1.]]), blocksize=(1,1))
        P_I = bsr_matrix( array([[1.], [0.]]), blocksize=(1,1) )
        assert_equal(array([0]),  params['Cpts'])
        assert_equal(array([1]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        Cpts = array([0,1])
        AggOp = csr_matrix(array([[1.,0], [0.,1.]]))
        T = AggOp.copy().tobsr()
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[1., 0.],[0., 1.]]), blocksize=(1,1))
        I_F = bsr_matrix( array([[0., 0.],[0., 0.]]), blocksize=(1,1))
        P_I = bsr_matrix( array([[1., 0.], [0., 1.]]), blocksize=(1,1) )
        assert_equal(array([0,1]),  params['Cpts'])
        assert_equal(array([ ]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        Cpts = array([ ])
        AggOp = csr_matrix(array([[0.], [0.]]))
        T = AggOp.copy().tobsr()
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[0., 0.],[0., 0.]]), blocksize=(1,1))
        I_F = bsr_matrix( array([[1., 0.],[0., 1.]]), blocksize=(1,1))
        P_I = bsr_matrix( array([[ 0.], [0. ]]), blocksize=(1,1) )
        assert_equal(array([ ]),  params['Cpts'])
        assert_equal(array([0,1]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        A = A.tobsr( blocksize=(2,2) )
        Cpts = array([0])
        AggOp = csr_matrix(array([[1.]]) )
        T = bsr_matrix(array([[1., 1.], [1., 2.]]), blocksize=(2,2))
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[1., 0.],[0., 1.]]), blocksize=(2,2))
        I_F = bsr_matrix( array([[0., 0.],[0., 0.]]), blocksize=(2,2))
        P_I = bsr_matrix( array([[1., 0.],[0., 1.]]), blocksize=(2,2))
        assert_equal(array([0,1]),  params['Cpts'])
        assert_equal(array([ ]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)
        ##
        Cpts = array([ ])
        AggOp = csr_matrix(array([[1.]]) )
        T = bsr_matrix(array([[1., 1.], [1., 2.]]), blocksize=(2,2))
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix( array([[0., 0.],[0., 0.]]), blocksize=(2,2))
        I_F = bsr_matrix( array([[1., 0.],[0., 1.]]), blocksize=(2,2))
        P_I = bsr_matrix( array([[0., 0.],[0., 0.]]), blocksize=(2,2))
        assert_equal(array([ ]),  params['Cpts'])
        assert_equal(array([0,1]),  params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)

        ##
        # Begin more "realistic" tests
        A = poisson((10,), format='csr')
        Cpts = array([3, 7])
        AggOp = ([[ 1., 0.],
                  [ 1., 0.],
                  [ 1., 0.],
                  [ 1., 0.],
                  [ 1., 0.],
                  [ 0., 1.],
                  [ 0., 1.],
                  [ 0., 1.],
                  [ 0., 1.],
                  [ 0., 1.]])
        AggOp = csr_matrix(AggOp)
        T = AggOp.copy().tobsr()

        ##
        # CSR Test
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix(( array([[[ 1.]], [[ 1.]]]), 
                array([3, 7]), array([0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2]) ), 
                shape=(10,10) )
        I_F = bsr_matrix(( 
              array([[[ 1.]], [[ 1.]], [[ 1.]], [[ 1.]], [[ 1.]], [[ 1.]], [[ 1.]], [[ 1.]]]), 
              array([0, 1, 2, 4, 5, 6, 8, 9]), 
              array([0, 1, 2, 3, 3, 4, 5, 6, 6, 7, 8]) ), 
              shape=(10,10) )
        P_I = matrix([[ 0.,  0.],
                      [ 0.,  0.],
                      [ 0.,  0.],
                      [ 1.,  0.],
                      [ 0.,  0.],
                      [ 0.,  0.],
                      [ 0.,  0.],
                      [ 0.,  1.],
                      [ 0.,  0.],
                      [ 0.,  0.]]) 
        P_I = bsr_matrix(P_I, blocksize=(1,1))
        Fpts = array([0,1,2,4,5,6,8,9])
        assert_equal(Cpts, params['Cpts'])
        assert_equal(Fpts, params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)

        ##
        # BSR Test
        A = A.tobsr(blocksize=(2,2)) 
        Cpts = array([1, 3])
        AggOp = ([[ 1., 0.],
                  [ 1., 0.],
                  [ 1., 0.],
                  [ 0., 1.],
                  [ 0., 1.]])
        AggOp = csr_matrix(AggOp)
        T = hstack((T.todense(), T.todense()))[:,[0,2,1,3]]
        T = bsr_matrix(T, blocksize=(2,2)) 
        params = get_Cpt_params(A, Cpts, AggOp, T)
        I_C = bsr_matrix(( array([ [[ 1.,  0.],[ 0.,  1.]],
                                   [[ 1.,  0.],[ 0.,  1.]]]), 
                array([1, 3]), 
                array([0, 0, 1, 1, 2, 2]) ), 
                shape=(10,10) )
        I_F = bsr_matrix(( 
              array([[[ 1.,  0.],[ 0.,  1.]], 
                     [[ 1.,  0.],[ 0.,  1.]], 
                     [[ 1.,  0.],[ 0.,  1.]]]), 
              array([0, 2, 4]), 
              array([0, 1, 1, 2, 2, 3]) ), 
              shape=(10,10) )
        P_I = matrix([[ 0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.],
                      [ 1.,  0.,  0.,  0.],
                      [ 0.,  1.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.],
                      [ 0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  1.],
                      [ 0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.]]) 
        P_I = bsr_matrix(P_I, blocksize=(2,2))
        Fpts = array([0, 1, 4, 5, 8, 9])
        Cpts = array([2, 3, 6, 7])
        assert_equal(Cpts, params['Cpts'])
        assert_equal(Fpts, params['Fpts'])
        assert_equal(I_C.indptr,  params['I_C'].indptr)
        assert_equal(I_C.indices, params['I_C'].indices)
        assert_equal(I_C.data,    params['I_C'].data)
        assert_equal(I_F.indptr,  params['I_F'].indptr)
        assert_equal(I_F.indices, params['I_F'].indices)
        assert_equal(I_F.data,    params['I_F'].data)
        assert_equal(P_I.indptr,  params['P_I'].indptr)
        assert_equal(P_I.indices, params['P_I'].indices)
        assert_equal(P_I.data,    params['P_I'].data)

    def test_compute_BtBinv(self):
        ##
        # Trivially sized tests
        # 1x1x1
        T = matrix([[ 1.]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[1.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([ [[ 1. ]] ])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[0.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([ [[ 0. ]] ])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[0.5]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([ [[ 4. ]] ])
        assert_array_almost_equal(BtBinv, answer)
        ##
        # 2x1x1
        T = matrix([[ 1.,0.], [1.,1.]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[1.], [1.] ])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 1. ]], [[ 0.5]]])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.,0.], [1.,1.]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[0.], [1.] ])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 0. ]], [[ 1.]]])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.,0.], [1.,1.]])
        T = bsr_matrix(T, blocksize=(2,2))
        B = array([[0.], [2.] ])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 0.25 ]]])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.,  0.], [ 1.,  0.],
                    [ 0.,  .5], [ 0.,  .25]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[1.],[2.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 1.  ]], [[ 1.  ]],
                        [[ 0.25]], [[ 0.25]]])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.,  0.], [ 0.,  .25]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[1., 1.],[2., 1.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 0.25,  0.25], [ 0.25,  0.25]],
                        [[ 0.16,  0.08], [ 0.08,  0.04]]])
        assert_array_almost_equal(BtBinv, answer)
        ##
        T = matrix([[ 1.,  0.], [ 0.,  .25]])
        T = bsr_matrix(T, blocksize=(2,2))
        B = array([[1., 1.],[1., 1.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 0.125,  0.125],
                         [ 0.125,  0.125]]])
        assert_array_almost_equal(BtBinv, answer)

        ##
        # Simple BSR test
        T = matrix([[ 1.  ,  1.  ,  0.  ,  0.  ],
                    [ 1.  ,  1.  ,  0.  ,  0.  ],
                    [ 0.  ,  0.  ,  0.5 ,  0.5 ],
                    [ 0.  ,  0.  ,  0.25,  0.25]])
        T = bsr_matrix(T, blocksize=(2,2))
        B = array([[1., 1.],[1., 2.],[1., 1.],[1., 3.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 5. , -3. ], [-3. ,  2. ]],
                        [[ 2.5, -1. ], [-1. ,  0.5]]])
        assert_array_almost_equal(BtBinv, answer)

    def test_eliminate_diag_dom_nodes(self):
        ##
        # Simple CSR test
        from pyamg.gallery import poisson
        A = poisson( (4,), format='csr' )
        C = eliminate_diag_dom_nodes(A, A.copy(), 1.1)
        answer = array([[ 1.,  0.,  0.,  0.],
            [ 0.,  2., -1.,  0.],
            [ 0., -1.,  2.,  0.],
            [ 0.,  0.,  0.,  1.]])
        assert_array_almost_equal(C.todense(), answer)
        
        ##
        # Simple BSR test
        from pyamg.gallery import poisson
        A = poisson( (6,), format='csr' )
        A.data[3] = 5.0
        A = A.tobsr( (2,2) )
        C = poisson( (3,), format='csr' )
        C = eliminate_diag_dom_nodes(A, C, 1.1)
        answer = array([[ 1.,  0.,  0.],
            [ 0.,  2., -1.],
            [ 0., -1.,  2.]])        
        assert_array_almost_equal(C.todense(), answer)
    
    def test_remove_diagonal(self):
        from pyamg.gallery import poisson
        from pyamg.util.utils import remove_diagonal
        A = poisson( (4,), format='csr' )
        C = remove_diagonal(A)
        exact = array([[ 0., -1.,  0.,  0.],
                       [-1.,  0., -1.,  0.],
                       [ 0., -1.,  0., -1.],
                       [ 0.,  0., -1.,  0.]])
        assert_array_almost_equal(C.todense(), exact)
    
    def test_scale_rows_by_largest_entry(self):
        from pyamg.gallery import poisson
        from pyamg.util.utils import scale_rows_by_largest_entry
        A = poisson( (4,), format='csr' )
        A.data = array(A.data, dtype=complex)
        A.data[1] = 8. 
        A = scale_rows_by_largest_entry(A)
        exact = array([[ 0.25,  1.  ,  0.  ,  0.  ],
                       [-0.5 ,  1.  , -0.5 ,  0.  ],
                       [ 0.  , -0.5 ,  1.  , -0.5 ],
                       [ 0.  ,  0.  , -0.5 ,  1.  ]])
        assert_array_almost_equal(A.todense(), exact)

class TestComplexUtils(TestCase):
    def test_diag_sparse(self):
        #check sparse -> array
        A = matrix([[-4-4.0j]])
        assert_equal(diag_sparse(csr_matrix(A)),[-4-4.0j])

        A = matrix([[1,0,-5],[-2,5-2.0j,0]])
        assert_equal(diag_sparse(csr_matrix(A)),[1,5-2.0j])

        #check array -> sparse
        A = matrix([[-4+1.0j]])
        assert_equal(diag_sparse(array([-4+1.0j])).todense(),csr_matrix(A).todense())

        A = matrix([[1,0],[0,5-2.0j]])
        assert_equal(diag_sparse(array([1,5-2.0j])).todense(),csr_matrix(A).todense())

    def test_symmetric_rescaling(self):
        cases = []
        A = array([ [ 5.5+1.0j,  3.5,    4.8   ],
                    [ 2. ,       9.9,  0.5-2.0j],
                    [ 6.5,       2.6,  5.7+1.0j]])
        A = csr_matrix( A )
        cases.append(A)
        P = diag_sparse([1,0,1.0j])
        cases.append( P*A*P )
        P = diag_sparse([0,1+1.0j,0])
        cases.append( P*A*P )

        for A in cases:
            D_sqrt,D_sqrt_inv,DAD = symmetric_rescaling(A)
            assert_almost_equal( diag_sparse(A) != 0, real(diag_sparse(DAD)) )
            assert_almost_equal( diag_sparse(DAD), D_sqrt*D_sqrt_inv )

            D_sqrt,D_sqrt_inv = diag_sparse(D_sqrt),diag_sparse(D_sqrt_inv)
            assert_almost_equal((D_sqrt_inv*A*D_sqrt_inv).todense(), DAD.todense())

    def test_symmetric_rescaling_sa(self):
        cases = []
        # case 1
        e = 1.0j*ones((5,1)).ravel()
        data = [ -1*e, 2*e, -1*e ]
        A = 1.0j*spdiags(data,[-1,0,1],5,5).tocsr()
        B = e.copy().reshape(-1,1)
        DAD_answer = array([[ 1. ,-0.5, 0. , 0. , 0. ],
                            [-0.5, 1. ,-0.5, 0. , 0. ],
                            [ 0. ,-0.5, 1. ,-0.5, 0. ],
                            [ 0. , 0. ,-0.5, 1. ,-0.5],
                            [ 0. , 0. , 0. ,-0.5, 1. ]])
        DB_answer = sqrt(2)*1.0j*e.reshape(-1,1)
        #            matrix   B    BH   expected matrix  expected B  expected BH 
        cases.append( (A,     B,  None,   DAD_answer,      DB_answer,   None) )

        for case in cases:
            [A,B,BH,DAD_answer,DB_answer,DBH_answer] = case
            
            [DAD, DB, DBH] = symmetric_rescaling_sa(A,B,BH=BH)
            assert_array_almost_equal(DAD.todense(), DAD_answer)
            assert_array_almost_equal(DB, DB_answer)
            if DBH_answer is not None:
                assert_array_almost_equal(DBH, DBH_answer)


    def test_get_diagonal(self):
        cases = []
        for i in range(1,6):
            A = rand(i,i)
            Ai = A + 1.0j*rand(i,i)
            cases.append(csr_matrix(A)) 
            cases.append(csr_matrix(Ai)) 


        for A in cases:
            D_A       = get_diagonal(A, norm_eq=False, inv=False)
            D_A_inv   = get_diagonal(A, norm_eq=False, inv=True)
            D_AA      = get_diagonal(A, norm_eq=1, inv=False)
            D_AA_inv  = get_diagonal(A, norm_eq=1, inv=True)
            D_AA2     = get_diagonal(A, norm_eq=2, inv=False)
            D_AA_inv2 = get_diagonal(A, norm_eq=2, inv=True)
            
            D = diag(A.todense())
            assert_almost_equal(D, D_A)
            D = 1.0/D
            assert_almost_equal(D, D_A_inv)
            
            D = diag((A.H*A).todense())
            assert_almost_equal(D, D_AA)
            D = 1.0/D
            assert_almost_equal(D, D_AA_inv)
            
            D = diag((A*A.H).todense())
            assert_almost_equal(D, D_AA2)
            D = 1.0/D
            assert_almost_equal(D, D_AA_inv2)


    def test_profile_solver(self):
        from scipy.sparse.linalg import cg
        from pyamg.gallery import poisson
        from pyamg.aggregation import smoothed_aggregation_solver

        A = poisson((100,100), format='csr')
        A.data = A.data + 1e-5*rand(A.nnz)
        ml = smoothed_aggregation_solver(A)

        opts = []
        opts.append( {} )
        opts.append( {'accel' : cg } )
        opts.append( {'accel' : cg, 'tol' : 1e-10 } )

        for kwargs in opts:
            residuals = profile_solver(ml, **kwargs)

    def test_to_type(self):
        w = 1.2
        x = ones((5,1))
        y = rand(3,2)
        z = csr_matrix(rand(2,2))
        inlist = [w, x, y, z]

        out = to_type(complex, inlist)
        for i in range(len(out)):
            assert( out[i].dtype==complex ) 
            if isspmatrix(out[i]):
                diff = ravel(out[i].data - inlist[i].data)
            else:
                diff = out[i] - inlist[i]
            assert_equal( max(abs(ravel(diff))), 0.0)

    def test_type_prep(self):
        w = 1.2
        x = ones((5,1))
        y = rand(3,2)
        z = csr_matrix(rand(2,2))
        inlist = [w, x, y, z]

        out = type_prep(complex, inlist)
        for i in range(len(out)):
            assert( out[i].dtype==complex ) 
            assert( not isscalar(out[i]) )
            if isspmatrix(out[i]):
                diff = ravel(out[i].data - inlist[i].data)
            else:
                diff = out[i] - inlist[i]
            assert_equal( max(abs(ravel(diff))), 0.0)
    
    def test_filter_operator(self):

        ##
        # Basic tests, with easy to compute answers
        # test one, the constant
        A = array([ [1.+0.j,1,1],[1,1,1],[0,1,0],[0,1,0],[0,0,1],[0,0,1]])
        C = array([ [1.+0.j,1,0],[1,1,0],[0,1,0],[0,1,0],[0,0,1],[0,0,1]])
        B = ones((3,1)) + 0.j
        Bf = ones((6,1)) + 1.0j * ones((6,1))
        A_filter = filter_operator(csr_matrix(A), csr_matrix(C), B, Bf).todense()
        A_known = matrix([[ 0.5+0.5j,  0.5+0.5j,  0.0+0.j ],
                          [ 0.5+0.5j,  0.5+0.5j,  0.0+0.j ],
                          [ 0.0+0.j ,  1.0+1.j ,  0.0+0.j ],
                          [ 0.0+0.j ,  1.0+1.j ,  0.0+0.j ],
                          [ 0.0+0.j ,  0.0+0.j ,  1.0+1.j ],
                          [ 0.0+0.j ,  0.0+0.j ,  1.0+1.j ]])
        assert_array_almost_equal(A_known, A_filter)
        
        ##
        # test two, the constant and linears
        # Note that for the rows with only one nonzero, Bf can't be
        # approximated exactly
        B = hstack( (B, arange(B.shape[0]).reshape(-1,1)) )
        Bf = hstack( (Bf, arange(Bf.shape[0]).reshape(-1,1) + 1.0j*arange(Bf.shape[0]).reshape(-1,1) ) )
        A_filter = filter_operator(csr_matrix(A), csr_matrix(C), B, Bf).todense()
        A_known = matrix([[ 1.0+1.j ,  0.0+0.j ,  0.0+0.j ],
                          [ 0.0+0.j ,  1.0+1.j ,  0.0+0.j ],
                          [ 0.0+0.j ,  1.5+1.5j,  0.0+0.j ],
                          [ 0.0+0.j ,  2.0+2.j ,  0.0+0.j ],
                          [ 0.0+0.j ,  0.0+0.j ,  1.8+1.8j],
                          [ 0.0+0.j ,  0.0+0.j ,  2.2+2.2j]])
        assert_array_almost_equal(A_known, A_filter)
        
    def test_scale_T(self):
        from scipy.sparse import csr_matrix, bsr_matrix
        from scipy import matrix

        ##
        # Test for one CSR and one BSR example
        T = matrix([[ 1.0,  0.,   0. ],
                    [ 0.5j, 0.,   0. ],
                    [ 0. ,  1.,   0. ],
                    [ 0. ,  .5j,  0. ],
                    [ 0. ,  0.,   1.j],
                    [ 0. ,  0.,   0.25 ]])
        T_answer = matrix([[ 0.-2.j ,  0.+0.j ,  0.+0.j ],
                           [ 1.+0.j ,  0.+0.j ,  0.+0.j ],
                           [ 0.+0.j ,  1.+0.j ,  0.+0.j ],
                           [ 0.+0.j ,  0.+0.5j,  0.+0.j ],
                           [ 0.+0.j ,  0.+0.j ,  0.+4.j ],
                           [ 0.+0.j ,  0.+0.j ,  1.+0.j ]])
        P_I = matrix([[ 0.,  0.,   0. ],
                      [ 1.,  0.,   0. ],
                      [ 0.,  1.,   0. ],
                      [ 0.,  0.,   0. ],
                      [ 0.,  0.,   0. ],
                      [ 0.,  0.,   1. ]])
        I_F = matrix([[ 1.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  1.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.]])
        T_scaled = scale_T(bsr_matrix(T), bsr_matrix(P_I), bsr_matrix(I_F)).todense()
        assert_array_almost_equal(T_answer, T_scaled)

        ##
        # BSR test
        T = matrix([[ 1.j, 1., 0.,  0. ],
                    [ 0.5, 1., 0.,  0. ],
                    [ 1. , 0., 0.,  0. ],
                    [ 0. , 1., 0.,  0. ],
                    [ 0. , 0., 2.j, 0. ],
                    [ 0. , 0., 0.,  1. ],
                    [ 0. , 0., 1.,  1. ],
                    [ 0. , 0., 1.,  1. ]])
        P_I = matrix([[ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.],
                      [ 1.,  0.,  0.,   0.],
                      [ 0.,  1.,  0.,   0.],
                      [ 0.,  0.,  1.,   0.],
                      [ 0.,  0.,  0.,   1.],
                      [ 0.,  0.,  0.,   0.],
                      [ 0.,  0.,  0.,   0.]])
        I_F = matrix([[ 1.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  1.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  1.,  0.],
                      [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  1.]])
        T_answer = matrix([[ 0.0+1.j ,  1.0+0.j ,  0.0+0.j ,  0.0+0.j ],
                           [ 0.5+0.j ,  1.0+0.j ,  0.0+0.j ,  0.0+0.j ],
                           [ 1.0+0.j ,  0.0+0.j ,  0.0+0.j ,  0.0+0.j ],
                           [ 0.0+0.j ,  1.0+0.j ,  0.0+0.j ,  0.0+0.j ],
                           [ 0.0+0.j ,  0.0+0.j ,  1.0+0.j ,  0.0+0.j ],
                           [ 0.0+0.j ,  0.0+0.j ,  0.0+0.j ,  1.0+0.j ],
                           [ 0.0+0.j ,  0.0+0.j ,  0.0-0.5j,  1.0+0.j ],
                           [ 0.0+0.j ,  0.0+0.j ,  0.0-0.5j,  1.0+0.j ]]) 
        T = bsr_matrix(T, blocksize=(2,2))
        P_I = bsr_matrix(P_I, blocksize=(2,2))
        I_F = bsr_matrix(I_F, blocksize=(2,2))
        T_scaled = scale_T(T, P_I, I_F).todense()
        assert_array_almost_equal(T_answer, T_scaled)
    
    def test_compute_BtBinv(self):
        from pyamg.gallery import poisson
        
        ##
        # Simple CSR test
        T = matrix([[ 1.j,  0.], [ 1.,  0.],
                    [ 0.,  .5], [ 0.,  .25]])
        T = bsr_matrix(T, blocksize=(1,1))
        B = array([[1.+1.j],[2.j]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 0.50+0.j]], [[ 0.50+0.j]],
                        [[ 0.25+0.j]], [[ 0.25+0.j]]])
        assert_array_almost_equal(BtBinv, answer)

        ##
        # Simple BSR test
        T = matrix([[ 1.  ,  0.  ,  0.   ,  1. ],
                    [ 1.  ,  0.  ,  0.   ,  1. ],
                    [ 0.  ,  0.  ,  0.5  ,  0. ],
                    [ 0.  ,  0.  ,  0.25 ,  0. ]])
        T = bsr_matrix(T, blocksize=(2,2))
        B = array([[1.j, 1.],[1.j, 3.],[1.j, 4.],[1.j, 2.]])
        BtBinv = compute_BtBinv(B, T)
        answer = array([[[ 1.5+0.j ,  0.0+0.5j], [ 0.0-0.5j,  0.2+0.j ]],
                        [[ 5.0+0.j ,  0.0+1.5j], [ 0.0-1.5j,  0.5+0.j ]]])
        assert_array_almost_equal(BtBinv, answer)
    
    def test_eliminate_diag_dom_nodes(self):
        ##
        # Simple CSR test
        from pyamg.gallery import poisson
        A = poisson( (4,), format='csr' )
        A.data = array(A.data, dtype=complex)
        A.data[0] = 1+5.0j
        C = eliminate_diag_dom_nodes(A, A.copy(), 1.1)
        answer = array([[ 1.,  0.,  0.,  0.],
            [ 0.,  2., -1.,  0.],
            [ 0., -1.,  2.,  0.],
            [ 0.,  0.,  0.,  1.]])
        assert_array_almost_equal(C.todense(), answer)
        
        ##
        # Simple BSR test
        from pyamg.gallery import poisson
        A = poisson( (6,), format='csr' )
        A.data = array(A.data, dtype=complex)
        A.data[3] = 1.0 + 5.0j
        A = A.tobsr( (2,2) )
        C = poisson( (3,), format='csr' )
        C = eliminate_diag_dom_nodes(A, C, 1.1)
        answer = array([[ 1.,  0.,  0.],
            [ 0.,  2., -1.],
            [ 0., -1.,  2.]])        
        assert_array_almost_equal(C.todense(), answer)

    def test_remove_diagonal(self):
        from pyamg.gallery import poisson
        from pyamg.util.utils import remove_diagonal
        A = poisson( (4,), format='csr' )
        C = remove_diagonal(1.0j*A)
        exact = array([[ 0., -1.,  0.,  0.],
                       [-1.,  0., -1.,  0.],
                       [ 0., -1.,  0., -1.],
                       [ 0.,  0., -1.,  0.]])
        assert_array_almost_equal(C.todense(), 1.0j*exact)

    def test_scale_rows_by_largest_entry(self):
        from pyamg.gallery import poisson
        from pyamg.util.utils import scale_rows_by_largest_entry
        A = poisson( (4,), format='csr' )
        A.data = array(A.data, dtype=complex)
        A.data[1] = 3. + 2.j
        A = scale_rows_by_largest_entry(A)
        exact = array( [[ 0.55470020+0.j,  0.83205029+0.5547002j,  0.0,      0.0],
                        [-0.50000000+0.j,  1.00000000+0.j,        -0.5+0.j,  0.0+0.j],
                        [ 0.00000000+0.j, -0.50000000+0.j,         1.0+0.j, -0.5+0.j],
                        [ 0.00000000+0.j,  0.00000000+0.j,        -0.5+0.j,  1.0+0.j]])
        assert_array_almost_equal(A.todense(), exact)



    ## JBS: no explicitly complex tests necessary for get_Cpt_params
    ## def test_get_Cpt_params(self):

class TestLevelize(TestCase):
    
    def test_levelize_smooth_or_improve_candidates(self):
        from pyamg.util.utils import levelize_smooth_or_improve_candidates
        # test 1
        result = levelize_smooth_or_improve_candidates([('jacobi', {})], 5)
        assert_equal(result, [('jacobi', {}) for i in range(5)])
        # test 2
        result = levelize_smooth_or_improve_candidates('jacobi', 5)
        assert_equal(result, ['jacobi' for i in range(5)])
        # test 3
        result = levelize_smooth_or_improve_candidates(('jacobi', {}), 5)
        assert_equal(result, [('jacobi', {}) for i in range(5)])
        ## test 4
        result = levelize_smooth_or_improve_candidates([('jacobi', {}),None], 5)
        assert_equal(result, [('jacobi', {}), None, None, None, None]) 

    def test_levelize_strength_or_aggregation(self):
        from pyamg.util.utils import levelize_strength_or_aggregation
        from pyamg.gallery import poisson
        A = poisson( (100,), format='csr')
        # test 1
        max_levels, max_coarse, result = levelize_strength_or_aggregation([('symmetric', {})], 5, 5)
        assert_equal(result, [('symmetric', {}) for i in range(4)])
        assert_equal(max_levels, 5)
        assert_equal(max_coarse, 5)
        # test 2
        max_levels, max_coarse, result = levelize_strength_or_aggregation('symmetric', 5, 5)
        assert_equal(result, ['symmetric' for i in range(4)])
        assert_equal(max_levels, 5)
        assert_equal(max_coarse, 5)
        # test 3
        max_levels, max_coarse, result = levelize_strength_or_aggregation(('symmetric', {}), 5, 5)
        assert_equal(result, [('symmetric', {}) for i in range(4)])
        assert_equal(max_levels, 5)
        assert_equal(max_coarse, 5)
        # test 4
        max_levels, max_coarse, result = levelize_strength_or_aggregation([('symmetric', {}),None], 5, 5)
        assert_equal(result, [('symmetric', {}), None, None, None]) 
        assert_equal(max_levels, 5)
        assert_equal(max_coarse, 5)
        # test 5
        max_levels, max_coarse, result = levelize_strength_or_aggregation(('predefined',{'C' : A}), 5, 5)
        assert_array_equal(result, [('predefined',{'C' : A})])
        assert_equal(max_levels, 2)
        assert_equal(max_coarse, 0)
        # test 6
        max_levels, max_coarse, result = levelize_strength_or_aggregation([('predefined',{'C' : A}), \
                                                                ('predefined',{'C' : A})], 5, 5)
        assert_array_equal(result, [('predefined',{'C' : A}), ('predefined',{'C' : A})])
        assert_equal(max_levels, 3)
        assert_equal(max_coarse, 0)
        # test 7
        max_levels, max_coarse, result = levelize_strength_or_aggregation(None, 5, 5)
        assert_equal(result, [(None,{}) for i in range(4)])
        assert_equal(max_levels, 5)
        assert_equal(max_coarse, 5)






from pyamg.testing import *

import numpy
import scipy
from scipy.sparse import spdiags, csr_matrix, bsr_matrix, eye
from scipy import arange, ones, zeros, array, allclose, zeros_like, \
        tril, diag, triu, rand, asmatrix, mat
from scipy.linalg import solve

from pyamg.gallery    import poisson, sprand, elasticity
from pyamg.relaxation import *
from pyamg.util.utils import get_block_diag

# Ignore efficiency warnings
import warnings
from scipy.sparse import SparseEfficiencyWarning
warnings.simplefilter('ignore',SparseEfficiencyWarning)

class TestCommonRelaxation(TestCase):
    def setUp(self):
        self.cases = []
        self.cases.append( (gauss_seidel,              (),               {}) )
        self.cases.append( (jacobi,                    (),               {}) )
        self.cases.append( (block_jacobi,              (),               {}) )
        self.cases.append( (block_gauss_seidel,        (),               {}) )
        self.cases.append( (jacobi_ne,                 (),               {}) )
        self.cases.append( (schwarz,                   (),               {}) )
        self.cases.append( (sor,                       (0.5,),           {}) )
        self.cases.append( (gauss_seidel_indexed,      ([1,0],),         {}) )
        self.cases.append( (polynomial,                ([0.6,0.1],),     {}) )


    def test_single_precision(self):

        for method,args,kwargs in self.cases:
            A = poisson((4,), format='csr').astype('float32')
            b = arange(A.shape[0], dtype='float32')
            x = 0*b
            method(A, x, b, *args, **kwargs)


    def test_double_precision(self):

        for method,args,kwargs in self.cases:
            A = poisson((4,), format='csr').astype('float64')
            b = arange(A.shape[0], dtype='float64')
            x = 0*b
            method(A, x, b, *args, **kwargs)


    def test_strided_x(self):
        """non-contiguous x should raise errors"""

        for method,args,kwargs in self.cases:
            A = poisson((4,), format='csr').astype('float64')
            b = arange(A.shape[0], dtype='float64')
            x = zeros(2*A.shape[0])[::2]
            assert_raises(ValueError, method, A, x, b, *args, **kwargs)


    def test_mixed_precision(self):
        """mixed precision arguments should raise errors"""

        for method,args,kwargs in self.cases:
            A32 = poisson((4,), format='csr').astype('float32')
            b32 = arange(A32.shape[0], dtype='float32')
            x32 = 0*b32

            A64 = poisson((4,), format='csr').astype('float64')
            b64 = arange(A64.shape[0], dtype='float64')
            x64 = 0*b64

            assert_raises(TypeError, method, A32, x32, b64, *args, **kwargs)
            assert_raises(TypeError, method, A32, x64, b32, *args, **kwargs)
            assert_raises(TypeError, method, A64, x32, b32, *args, **kwargs)
            assert_raises(TypeError, method, A32, x64, b64, *args, **kwargs)
            assert_raises(TypeError, method, A64, x64, b32, *args, **kwargs)
            assert_raises(TypeError, method, A64, x32, b64, *args, **kwargs)


    def test_vector_sizes(self):
        """incorrect vector sizes should raise errors"""

        for method,args,kwargs in self.cases:
            A = poisson((4,), format='csr').astype('float64')
            b4 = arange(4, dtype='float64')
            x4 = 0*b4
            b5 = arange(5, dtype='float64')
            x5 = 0*b5

            assert_raises(ValueError, method, A, x4, b5, *args, **kwargs)
            assert_raises(ValueError, method, A, x5, b4, *args, **kwargs)
            assert_raises(ValueError, method, A, x5, b5, *args, **kwargs)

    def test_non_square_matrix(self):

        for method,args,kwargs in self.cases:
            A = poisson((4,), format='csr').astype('float64')
            A = A[:3]
            b = arange(A.shape[0], dtype='float64')
            x = ones(A.shape[1], dtype='float64')

            assert_raises(ValueError, method, A, x, b, *args, **kwargs)

class TestRelaxation(TestCase):
    def test_polynomial(self):
        N  = 3
        A  = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x0 = arange(N, dtype=A.dtype)
        x  = x0.copy()
        b  = zeros(N, dtype=A.dtype)

        r = (b - A*x0)
        polynomial(A,x,b,[-1.0/3.0])
        assert_almost_equal(x,x0-1.0/3.0*r)

        x = x0.copy()
        polynomial(A,x,b,[0.2,-1])
        assert_almost_equal(x,x0 + 0.2*A*r - r)

        x = x0.copy()
        polynomial(A,x,b,[0.2,-1])
        assert_almost_equal(x,x0 + 0.2*A*r - r)

        x = x0.copy()
        polynomial(A,x,b,[-0.14285714,  1., -2.])
        assert_almost_equal(x,x0 - 0.14285714*A*A*r + A*r - 2*r)

        # polynomial() optimizes for the case x=0
        x = 0*x0
        polynomial(A,x,b,[-1.0/3.0])
        assert_almost_equal(x,1.0/3.0*b)

        x = 0*x0
        polynomial(A,x,b,[-0.14285714,  1., -2.])
        assert_almost_equal(x,0.14285714*A*A*b + A*b - 2*b)


    def test_jacobi(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        jacobi(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = zeros(N)
        b = arange(N).astype(numpy.float64)
        jacobi(A,x,b)
        assert_almost_equal(x,array([0.0,0.5,1.0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        jacobi(A,x,b)
        assert_almost_equal(x,array([0.5,1.0,0.5]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10], dtype=A.dtype)
        jacobi(A,x,b)
        assert_almost_equal(x,array([5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10,20,30], dtype=A.dtype)
        jacobi(A,x,b)
        assert_almost_equal(x,array([5.5,11.0,15.5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        x_copy = x.copy()
        b = array([10,20,30], dtype=A.dtype)
        jacobi(A,x,b,omega=1.0/3.0)
        assert_almost_equal(x,2.0/3.0*x_copy + 1.0/3.0*array([5.5,11.0,15.5]))


    def test_jacobi_bsr(self):
        cases = []
        #JBS: remove some N
        for N in [1,2,3,4,5,6,10]:
            cases.append( spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr() )
            cases.append( elasticity.linear_elasticity((N,N))[0].tocsr() )
            C = csr_matrix( rand(N,N) )
            cases.append( C*C.H )
            C = sprand(N*2,N*2,0.3) + eye(N*2,N*2)
            cases.append( C*C.H )

        for A in cases:
            divisors = [ n for n in range(1,A.shape[0]+1) if A.shape[0] % n == 0 ]

            x_csr = arange(A.shape[0]).astype(numpy.float64)
            b = x_csr**2
            jacobi(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr = arange(B.shape[0]).astype(numpy.float64)
                jacobi(B,x_bsr,b)
                assert_almost_equal(x_bsr,x_csr)


    def test_gauss_seidel_bsr(self):
        sweeps = ['forward', 'backward', 'symmetric']
        cases = []
        for N in [1,2,3,4,5,6,10]:
            cases.append( spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr() )
            cases.append( elasticity.linear_elasticity((N,N))[0].tocsr() )
            C = csr_matrix( rand(N,N) )
            cases.append( C*C.H )
            C = sprand(N*2,N*2,0.3) + eye(N*2,N*2)
            cases.append( C*C.H )


        for A in cases:
            for sweep in sweeps:
                divisors = [ n for n in range(1,A.shape[0]+1) if A.shape[0] % n == 0 ]

                x_csr = arange(A.shape[0]).astype(numpy.float64)
                b = x_csr**2
                gauss_seidel(A,x_csr,b,sweep=sweep)

                for D in divisors:
                    B = A.tobsr(blocksize=(D,D))
                    x_bsr = arange(B.shape[0]).astype(numpy.float64)
                    gauss_seidel(B,x_bsr,b,sweep=sweep)
                    assert_almost_equal(x_bsr,x_csr)

    def test_gauss_seidel_gold(self):
        scipy.random.seed(0)

        cases = []
        cases.append( poisson( (4,), format='csr' ) )
        cases.append( poisson( (4,4), format='csr' ) )

        temp = asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.T * temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = A.todense()

            L = asmatrix(tril(A,k=-1))
            D = asmatrix(diag(diag(A)))
            U = asmatrix(triu(A,k=1))

            for i in range(iterations):
                if sweep == 'forward':
                    x = solve(L + D, (b - U*x) )
                elif sweep == 'backward':
                    x = solve(U + D, (b - L*x) )
                else:
                    x = solve(L + D, (b - U*x) )
                    x = solve(U + D, (b - L*x) )
            return x


        for A in cases:

            b = asmatrix(rand(A.shape[0],1))
            x = asmatrix(rand(A.shape[0],1))

            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )



    def test_gauss_seidel_csr(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([1.0/2.0,5.0/4.0,5.0/8.0]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel(A,x,b,sweep='backward')
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel(A,x,b,sweep='backward')
        assert_almost_equal(x,array([1.0/8.0,1.0/4.0,1.0/2.0]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10], dtype=A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10,20,30], dtype=A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([11.0/2.0,55.0/4,175.0/8.0]))


        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = ones(N)
        b = zeros(N)
        gauss_seidel(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)
        x = ones(N)
        gauss_seidel(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.01 and resid2 < 0.01)
        self.assert_(allclose(resid1,resid2))

    def test_gauss_seidel_indexed(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[0])
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[0,1,2])
        assert_almost_equal(x,array([1.0/2.0,5.0/4.0,5.0/8.0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[2,1,0],sweep='backward')
        assert_almost_equal(x,array([1.0/2.0,5.0/4.0,5.0/8.0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[0,1,2],sweep='backward')
        assert_almost_equal(x,array([1.0/8.0,1.0/4.0,1.0/2.0]))

        N = 4
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = ones(N)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[0,3])
        assert_almost_equal(x,array([1.0/2.0, 1.0, 1.0, 1.0/2.0]))

        N = 4
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = ones(N)
        b = zeros(N)
        gauss_seidel_indexed(A,x,b,[0,0])
        assert_almost_equal(x,array([1.0/2.0, 1.0, 1.0, 1.0]))

    def test_jacobi_ne(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = zeros(N)
        b = arange(N).astype(numpy.float64)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([-1./6., -1./15., 19./30.]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([2./5., 7./5., 4./5.]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10], dtype=A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10,20,30], dtype=A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([16./15., 1./15., (9 + 7./15.) ]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        x_copy = x.copy()
        b = array([10,20,30], dtype=A.dtype)
        jacobi_ne(A,x,b,omega=1.0/3.0)
        assert_almost_equal(x,2.0/3.0*x_copy + 1.0/3.0*array([16./15., 1./15., (9 + 7./15.) ]))

    def test_gauss_seidel_ne_bsr(self):
        cases = []

        #JBS: remove some N
        for N in [1,2,3,4,5,6,10]:
            A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()

            divisors = [ n for n in range(1,N+1) if N % n == 0 ]

            x_csr = arange(N).astype(numpy.float64)
            b = x_csr**2
            gauss_seidel_ne(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr = arange(N).astype(numpy.float64)
                gauss_seidel_ne(B,x_bsr,b)
                assert_almost_equal(x_bsr,x_csr)

    def test_gauss_seidel_ne_new(self):
        scipy.random.seed(0)

        cases = []
        cases.append( poisson( (4,), format='csr' ) )
        cases.append( poisson( (4,4), format='csr' ) )

        temp = asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.T * temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = mat(A.todense())
            AA = A*A.T

            L = asmatrix(tril(AA,k=0))
            U = asmatrix(triu(AA,k=0))

            for i in range(iterations):
                if sweep == 'forward':
                    x = x + A.T*(solve(L, (b - A*x) ))
                elif sweep == 'backward':
                    x = x + A.T*(solve(U, (b - A*x) ))
                else:
                    x = x + A.T*(solve(L, (b - A*x) ))
                    x = x + A.T*(solve(U, (b - A*x) ))
            return x

        for A in cases:

            b = asmatrix(rand(A.shape[0],1))
            x = asmatrix(rand(A.shape[0],1))

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )



    def test_gauss_seidel_ne_csr(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(numpy.float64)
        b = zeros(N)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([4./15., 8./5., 4./5.]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = zeros(N, dtype=A.dtype)
        gauss_seidel_ne(A,x,b,sweep='backward')
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = zeros(N, dtype=A.dtype)
        gauss_seidel_ne(A,x,b,sweep='backward')
        assert_almost_equal(x,array([2./5., 4./5., 6./5.]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10], dtype=A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([5]))

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10], dtype=A.dtype)
        gauss_seidel_ne(A,x,b,sweep='backward')
        assert_almost_equal(x,array([5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N, dtype=A.dtype)
        b = array([10,20,30], dtype=A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([-2./5., -2./5., (14 + 4./5.)]))

        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = ones(N)
        b = zeros(N)
        gauss_seidel_ne(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)
        x = ones(N)
        gauss_seidel_ne(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.2 and resid2 < 0.2)
        self.assert_(allclose(resid1,resid2))

    def test_gauss_seidel_nr_bsr(self):
        cases = []

        for N in [1,2,3,4,5,6,10]:
            A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()

            divisors = [ n for n in range(1,N+1) if N % n == 0 ]

            x_csr = arange(N).astype(numpy.float64)
            b = x_csr**2
            gauss_seidel_nr(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr = arange(N).astype(numpy.float64)
                gauss_seidel_nr(B,x_bsr,b)
                assert_almost_equal(x_bsr,x_csr)

    def test_gauss_seidel_nr(self):
        scipy.random.seed(0)

        cases = []
        cases.append( poisson( (4,), format='csr' ) )
        cases.append( poisson( (4,4), format='csr' ) )

        temp = asmatrix( rand(1,1) )
        cases.append( csr_matrix( temp) )
        temp = asmatrix( rand(2,2) )
        cases.append( csr_matrix( temp) )
        temp = asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = mat(A.todense())
            AA = A.H*A

            L = asmatrix(tril(AA,k=0))
            U = asmatrix(triu(AA,k=0))

            for i in range(iterations):
                if sweep == 'forward':
                    x = x + (solve(L, A.H*(b - A*x) ))
                elif sweep == 'backward':
                    x = x + (solve(U, A.H*(b - A*x) ))
                else:
                    x = x + (solve(L, A.H*(b - A*x) ))
                    x = x + (solve(U, A.H*(b - A*x) ))
            return x

        for A in cases:

            b = asmatrix(rand(A.shape[0],1))
            x = asmatrix(rand(A.shape[0],1))

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )

        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = ones(N)
        b = zeros(N)
        gauss_seidel_nr(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)
        x = ones(N)
        gauss_seidel_nr(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.2 and resid2 < 0.2)
        self.assert_(allclose(resid1,resid2))


    def test_schwarz_gold(self):
        scipy.random.seed(0)

        cases = []
        cases.append( poisson( (4,), format='csr' ) )
        cases.append( poisson( (4,4), format='csr' ) )
        A = poisson( (8,8), format='csr' )
        A.data[0] = 10.0; A.data[1] = -0.5; A.data[3] = -0.5
        cases.append( A )

        temp = asmatrix( rand(1,1) )
        cases.append( csr_matrix( temp.T * temp) )
        temp = asmatrix( rand(2,2) )
        cases.append( csr_matrix( temp.T * temp) )
        temp = asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.T * temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep='forward'):
            A = csr_matrix(A)

            ##
            # Default is point-wise iteration with each subdomain a point's neighborhood
            # in the matrix graph
            subdomains = [A.indices[A.indptr[i]:A.indptr[i+1]] for i in range(A.shape[0]) ]

            ##
            # extract each subdomain's block from the matrix
            subblocks = [ scipy.linalg.pinv2(( (A[subdomains[i],:]).tocsc()[:,subdomains[i]] ).todense())\
                          for i in range(len(subdomains)) ]

            if sweep == 'forward':
                indices = range(len(subdomains))
            elif sweep == 'backward':
                indices = range(len(subdomains)-1,-1,-1)
            elif sweep == 'symmetric':
                indices = range(len(subdomains))
                indices.extend( range(len(subdomains)-1,-1,-1) )

            ##
            # Multiplicative Schwarz iterations
            for j in xrange(iterations):
                for i in indices:
                    x[subdomains[i]] = scipy.dot(subblocks[i], (b[subdomains[i]] - A[subdomains[i],:]*x)) + \
                                       x[subdomains[i]]

            return x

        for A in cases:

            b = asmatrix(rand(A.shape[0],1))
            x = asmatrix(rand(A.shape[0],1))

            x_copy = x.copy()
            schwarz(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            schwarz(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            schwarz(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )


# Test complex arithmetic
class TestComplexRelaxation(TestCase):
    def test_jacobi(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = zeros(N).astype(A.dtype)
        jacobi(A,x,b)
        assert_almost_equal(x,array([0]))

        x = array([1.0 + 1.0j])
        b = array([-1.0 + 1.0j])
        omega = 4.0 - 1.0j
        jacobi(A,x,b,omega=omega)
        assert_almost_equal(x,array([-3.5]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = zeros(N).astype(A.dtype)
        b = arange(N).astype(A.dtype)
        b = b + 1.0j*b
        soln = array([0.0,0.5,1.0])
        jacobi(A,x,b)
        assert_almost_equal(x,soln)

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        soln = array([0.5 + 0.5j,1.0 + 1.0j,0.5+0.5j])
        jacobi(A,x,b)
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = array([10]).astype(A.dtype)
        jacobi(A,x,b)
        assert_almost_equal(x,array([2.5 - 2.5j]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = array([10,20,30]).astype(A.dtype)
        soln = array([3.0-2.0j, 6.0-4.0j, 8.0-7.0j])
        jacobi(A,x,b)
        assert_almost_equal(x,soln)

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        x_copy = x.copy()
        b = array([10,20,30]).astype(A.dtype)
        soln = 2.0/3.0*x_copy + 1.0/3.0*array([3.0-2.0j, 6.0-4.0j, 8.0-7.0j])
        jacobi(A,x,b,omega=1.0/3.0)
        assert_almost_equal(x,soln)

    def test_jacobi_bsr(self):
        cases = []
        for N in [1,2,3,4,5,6,10]:
            #
            C = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()
            C.data = C.data + 1.0j*1e-3*rand(C.data.shape[0],)
            cases.append( C )
            #
            cases.append( 1.0j*elasticity.linear_elasticity((N,N))[0].tocsr() )
            #
            C = csr_matrix( rand(N,N) + 1.0j*rand(N,N) )
            cases.append( C*C.H )
            #
            C = sprand(N*2,N*2,0.3) + 1.0j*sprand(N*2,N*2,0.3) + eye(N*2,N*2)
            cases.append( C*C.H )


        for A in cases:
            divisors = [ n for n in range(1,A.shape[0]+1) if A.shape[0] % n == 0 ]

            x0 = (arange(A.shape[0]) + 1.0j*1e-3*rand(A.shape[0],)).astype(A.dtype)
            x_csr = x0.copy()
            b = x_csr**2
            jacobi(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr = x0.copy()
                jacobi(B,x_bsr,b)
                assert_almost_equal(x_bsr,x_csr)


    def test_gauss_seidel_bsr(self):
        sweeps = ['forward', 'backward', 'symmetric']
        cases = []
        for N in [1,2,3,4,5,6,10]:
            #
            C = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()
            C.data = C.data + 1.0j*1e-3*rand(C.data.shape[0],)
            cases.append( C )
            #
            cases.append( 1.0j*elasticity.linear_elasticity((N,N))[0].tocsr() )
            #
            C = csr_matrix( rand(N,N) + 1.0j*rand(N,N) )
            cases.append( C*C.H )
            #
            C = sprand(N*2,N*2,0.3) + 1.0j*sprand(N*2,N*2,0.3) + eye(N*2,N*2)
            cases.append( C*C.H )


        for A in cases:
            for sweep in sweeps:
                divisors = [ n for n in range(1,A.shape[0]+1) if A.shape[0] % n == 0 ]

                x0 = (arange(A.shape[0]) + 1.0j*1e-3*rand(A.shape[0],)).astype(A.dtype)
                x_csr = x0.copy()
                b = x_csr**2
                gauss_seidel(A,x_csr,b,sweep=sweep)

                for D in divisors:
                    B = A.tobsr(blocksize=(D,D))
                    x_bsr = x0.copy()
                    gauss_seidel(B,x_bsr,b,sweep=sweep)
                    assert_almost_equal(x_bsr,x_csr)


    def test_schwarz_gold(self):
        scipy.random.seed(0)

        cases = []
        A = poisson( (4,), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A)
        A = poisson( (4,4), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A);

        temp = asmatrix( rand(1,1) ) + 1.0j*asmatrix( rand(1,1) )
        cases.append( csr_matrix( temp.H * temp) )
        temp = asmatrix( rand(2,2) ) + 1.0j*asmatrix( rand(2,2) )
        cases.append( csr_matrix( temp.H * temp) )
        temp = asmatrix( rand(4,4) ) + 1.0j*asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.H * temp) )

        # reference implementation
        def gold(A,x,b,iterations):
            A = csr_matrix(A)
            ##
            # Default is point-wise iteration with each subdomain a point's neighborhood
            # in the matrix graph
            subdomains = [A.indices[A.indptr[i]:A.indptr[i+1]] for i in range(A.shape[0]) ]
            ##
            # extract each subdomain's block from the matrix
            subblocks = [ scipy.linalg.pinv2(( (A[subdomains[i],:]).tocsc()[:,subdomains[i]] ).todense())\
                          for i in range(len(subdomains)) ]
            ##
            # Multiplicative Schwarz iterations
            for j in xrange(iterations):
                for i in xrange(len(subdomains)):
                    x[subdomains[i]] = scipy.dot(subblocks[i], (b[subdomains[i]] - A[subdomains[i],:]*x)) + \
                                       x[subdomains[i]]
            return x


        for A in cases:

            b = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)
            x = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)

            x_copy = x.copy()
            schwarz(A, x, b, iterations=1)
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1) )

    def test_gauss_seidel_new(self):
        scipy.random.seed(0)

        cases = []
        A = poisson( (4,), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A)
        A = poisson( (4,4), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A);

        temp = asmatrix( rand(4,4) ) + 1.0j*asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.H * temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = A.todense()

            L = asmatrix(tril(A,k=-1))
            D = asmatrix(diag(diag(A)))
            U = asmatrix(triu(A,k=1))

            for i in range(iterations):
                if sweep == 'forward':
                    x = solve(L + D, (b - U*x) )
                elif sweep == 'backward':
                    x = solve(U + D, (b - L*x) )
                else:
                    x = solve(L + D, (b - U*x) )
                    x = solve(U + D, (b - L*x) )
            return x


        for A in cases:

            b = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)
            x = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)

            ##
            # Gauss-Seidel Tests
            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )

            ##
            # Indexed Gauss-Seidel Tests
            x_copy = x.copy()
            gauss_seidel_indexed(A, x, b, indices=arange(A.shape[0]), iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel_indexed(A, x, b, indices=arange(A.shape[0]), iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel_indexed(A, x, b, indices=arange(A.shape[0]), iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )


    def test_gauss_seidel_csr(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        soln = array([1.0/2.0,5.0/4.0,5.0/8.0])
        soln = soln + 1.0j*soln
        b = zeros(N).astype(A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel(A,x,b,sweep='backward')
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        soln = array([1.0/8.0,1.0/4.0,1.0/2.0])
        soln = soln + 1.0j*soln
        b = zeros(N).astype(A.dtype)
        gauss_seidel(A,x,b,sweep='backward')
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = array([10]).astype(A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,array([2.5 - 2.5j]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([3.0 - 2.0j, 7.5 - 5.0j, 11.25 - 10.0j])
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = array([10,20,30]).astype(A.dtype)
        gauss_seidel(A,x,b)
        assert_almost_equal(x,soln)

        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)
        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        gauss_seidel(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.03 and resid2 < 0.03)
        self.assert_(allclose(resid1,resid2))

    def test_jacobi_ne(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = zeros(N).astype(A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([-1./6., -1./15., 19./30.])
        x = zeros(N).astype(A.dtype)
        b = arange(N).astype(A.dtype)
        b = b + 1.0j*b
        jacobi_ne(A,x,b)
        assert_almost_equal(x,soln)

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([2./5. + 2.0j/5., 7./5. + 7.0j/5., 4./5. + 4.0j/5.])
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = array([10]).astype(A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,array([2.5 - 2.5j]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([11./15. + 1.0j/15., 11./15. + 31.0j/15, 77./15. - 53.0j/15.])
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = array([10,20,30]).astype(A.dtype)
        jacobi_ne(A,x,b)
        assert_almost_equal(x,soln)

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = array([10,20,30]).astype(A.dtype)
        x_copy = x.copy()
        soln = 2.0/3.0*x_copy + 1.0/3.0*array([11./15. + 1.0j/15., 11./15. + 31.0j/15, 77./15. - 53.0j/15.])
        jacobi_ne(A,x,b,omega=1.0/3.0)
        assert_almost_equal(x,soln)

    def test_gauss_seidel_ne_bsr(self):
        cases = []

        for N in [1,2,3,4,5,6,10]:
            A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()
            A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)

            divisors = [ n for n in range(1,N+1) if N % n == 0 ]

            x_csr = (arange(N) + 1.0j*1e-3*rand(N,)).astype(A.dtype)
            x_bsr = x_csr.copy()
            b = x_csr**2
            gauss_seidel_ne(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr_temp = x_bsr.copy()
                gauss_seidel_ne(B,x_bsr_temp,b)
                assert_almost_equal(x_bsr_temp,x_csr)

    def test_gauss_seidel_ne_new(self):
        scipy.random.seed(0)

        cases = []
        A = poisson( (4,), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A)
        A = poisson( (4,4), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A.tobsr(blocksize=(2,2)))

        temp = asmatrix( rand(4,4) ) + 1.0j*asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp.T * temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = mat(A.todense())
            AA = A*A.H

            L = asmatrix(tril(AA,k=0))
            U = asmatrix(triu(AA,k=0))

            for i in range(iterations):
                if sweep == 'forward':
                    x = x + A.H*(solve(L, (b - A*x) ))
                elif sweep == 'backward':
                    x = x + A.H*(solve(U, (b - A*x) ))
                else:
                    x = x + A.H*(solve(L, (b - A*x) ))
                    x = x + A.H*(solve(U, (b - A*x) ))
            return x


        for A in cases:

            b = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)
            x = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel_ne(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )



    def test_gauss_seidel_ne_csr(self):
        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        A.data = A.data + 1.0j*A.data
        b = zeros(N).astype(A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([4./15., 8./5., 4./5.])
        soln = soln + 1.0j*soln
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel_ne(A,x,b,sweep='backward')
        assert_almost_equal(x,array([0]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([2./5., 4./5., 6./5.])
        soln = soln + 1.0j*soln
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel_ne(A,x,b,sweep='backward')
        assert_almost_equal(x,soln)

        N = 1
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = arange(N).astype(A.dtype)
        b = array([10]).astype(A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,array([2.5 - 2.5j]))

        N = 3
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        soln = array([-1./15.+0.6j,  0.6+2.6j,  7.8-6.2j])
        x = arange(N).astype(A.dtype)
        x = x + 1.0j*x
        b = array([10,20,30]).astype(A.dtype)
        gauss_seidel_ne(A,x,b)
        assert_almost_equal(x,soln)

        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel_ne(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)

        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        gauss_seidel_ne(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.3 and resid2 < 0.3)
        self.assert_(allclose(resid1,resid2))

    def test_gauss_seidel_nr_bsr(self):
        cases = []

        for N in [1,2,3,4,5,6,10]:
            A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N).tocsr()
            A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)

            divisors = [ n for n in range(1,N+1) if N % n == 0 ]

            x_csr = (arange(N) + 1.0j*1e-3*rand(N,)).astype(A.dtype)
            x_bsr = x_csr.copy()
            b = x_csr**2
            gauss_seidel_nr(A,x_csr,b)

            for D in divisors:
                B = A.tobsr(blocksize=(D,D))
                x_bsr_temp = x_bsr.copy()
                gauss_seidel_nr(B,x_bsr_temp,b)
                assert_almost_equal(x_bsr_temp,x_csr)

    def test_gauss_seidel_nr(self):
        scipy.random.seed(0)

        cases = []
        A = poisson( (4,), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A)
        A = poisson( (4,4), format='csr' ); A.data = A.data + 1.0j*1e-3*rand(A.data.shape[0],)
        cases.append(A.tobsr(blocksize=(2,2)))

        temp = asmatrix( rand(1,1) ) + 1.0j*asmatrix( rand(1,1) )
        cases.append( csr_matrix( temp) )
        temp = asmatrix( rand(2,2) ) + 1.0j*asmatrix( rand(2,2) )
        cases.append( csr_matrix( temp) )
        temp = asmatrix( rand(4,4) ) + 1.0j*asmatrix( rand(4,4) )
        cases.append( csr_matrix( temp) )

        # reference implementation
        def gold(A,x,b,iterations,sweep):
            A = mat(A.todense())
            AA = A.H*A

            L = asmatrix(tril(AA,k=0))
            U = asmatrix(triu(AA,k=0))

            for i in range(iterations):
                if sweep == 'forward':
                    x = x + (solve(L, A.H*(b - A*x) ))
                elif sweep == 'backward':
                    x = x + (solve(U, A.H*(b - A*x) ))
                else:
                    x = x + (solve(L, A.H*(b - A*x) ))
                    x = x + (solve(U, A.H*(b - A*x) ))
            return x


        for A in cases:

            b = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)
            x = asmatrix(rand(A.shape[0],1)) + 1.0j*asmatrix(rand(A.shape[0],1)).astype(A.dtype)

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='forward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='forward') )

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='backward')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='backward') )

            x_copy = x.copy()
            gauss_seidel_nr(A, x, b, iterations=1, sweep='symmetric')
            assert_almost_equal( x, gold(A,x_copy,b,iterations=1,sweep='symmetric') )

        #forward and backward passes should give same result with x=ones(N),b=zeros(N)
        N = 100
        A = spdiags([2*ones(N),-ones(N),-ones(N)],[0,-1,1],N,N,format='csr')
        A.data = A.data + 1.0j*A.data
        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        b = zeros(N).astype(A.dtype)
        gauss_seidel_nr(A,x,b,iterations=200,sweep='forward')
        resid1 = numpy.linalg.norm(A*x,2)

        x = ones(N).astype(A.dtype)
        x = x + 1.0j*x
        gauss_seidel_nr(A,x,b,iterations=200,sweep='backward')
        resid2 = numpy.linalg.norm(A*x,2)
        self.assert_(resid1 < 0.3 and resid2 < 0.3)
        self.assert_(allclose(resid1,resid2))


# Test both complex and real arithmetic
# for block_jacobi and block_gauss_seidel
class TestBlockRelaxation(TestCase):

    def test_block_jacobi(self):
        scipy.random.seed(0)

        # All real valued tests
        cases = []
        A = csr_matrix(scipy.zeros((1,1)))
        cases.append( (A,1) )
        A = csr_matrix(scipy.rand(1,1))
        cases.append( (A,1) )
        A = csr_matrix(scipy.zeros((2,2)))
        cases.append( (A,1) )
        cases.append( (A,2) )
        A = csr_matrix(scipy.rand(2,2))
        cases.append( (A,1) )
        cases.append( (A,2) )
        A = csr_matrix(scipy.zeros((3,3)))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = csr_matrix(scipy.rand(3,3))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = poisson( (4,4), format='csr')
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,4) )
        A = array([[ 9.1,  9.8,  9.6,  0. ,  3.6,  0. ],
                   [18.2, 19.6,  0. ,  0. ,  1.7,  2.8],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  4.2,  1. ,  1.1],
                   [ 0. ,  0. ,  9.1,  0. ,  0. ,  9.3]])
        A = csr_matrix(A)
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,3) )

        # reference implementation of 1 iteration
        def gold(A,x,b,blocksize,omega):

            A = csr_matrix(A)
            temp = x.copy()
            Dinv = get_block_diag(A, blocksize=blocksize, inv_flag=True)
            D = get_block_diag(A, blocksize=blocksize, inv_flag=False)
            A_no_D = A - bsr_matrix( (D, scipy.arange(Dinv.shape[0]), scipy.arange(Dinv.shape[0] + 1)), shape=A.shape)
            A_no_D = csr_matrix(A_no_D)

            for i in range(0, A.shape[0], blocksize):
                r = A_no_D[i:(i+blocksize),:]*temp
                r = scipy.mat(Dinv[i/blocksize,:,:])*scipy.mat(scipy.ravel(b[i:(i+blocksize)]) - scipy.ravel(r)).reshape(-1,1)
                x[i:(i+blocksize)] = (1.0 - omega)*temp[i:(i+blocksize)] + omega*scipy.ravel(r)

            return x


        for A,blocksize in cases:
            b = rand(A.shape[0])
            x = rand(A.shape[0])
            x_copy = x.copy()
            block_jacobi(A, x, b, blocksize=blocksize, iterations=1, omega=1.1)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 1.1), decimal=4 )

        # check for agreement between jacobi and block jacobi with blocksize=1
        A = poisson( (4,5), format='csr')
        b = rand(A.shape[0])
        x = rand(A.shape[0])
        x_copy = x.copy()
        block_jacobi(A, x, b, blocksize=1, iterations=2, omega=1.1)
        jacobi(A, x_copy, b, iterations=2, omega=1.1)
        assert_almost_equal( x, x_copy, decimal=4 )


        # complex valued tests
        cases = []
        A = csr_matrix(scipy.rand(3,3) + 1.0j*scipy.rand(3,3))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = poisson( (4,4), format='csr')
        A.data = A.data + 1.0j*scipy.rand(A.data.shape[0])
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,4) )
        A = array([[ 9.1j,  9.8j,  9.6,  0. ,  3.6,  0. ],
                   [18.2j, 19.6j,  0. ,  0. ,  1.7,  2.8],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  4.2, 1.0j,  1.1],
                   [ 0. ,  0. ,  9.1,  0. ,  0. ,  9.3]])
        A = csr_matrix(A)
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,3) )

        for A,blocksize in cases:
            b = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            x = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            x_copy = x.copy()
            block_jacobi(A, x, b, blocksize=blocksize, iterations=1, omega=1.1)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 1.1), decimal=4 )


    def test_block_gauss_seidel(self):
        scipy.random.seed(0)

        # All real valued tests
        cases = []
        A = csr_matrix(scipy.zeros((1,1)))
        cases.append( (A,1) )
        A = csr_matrix(scipy.rand(1,1))
        cases.append( (A,1) )
        A = csr_matrix(scipy.zeros((2,2)))
        cases.append( (A,1) )
        cases.append( (A,2) )
        A = csr_matrix(scipy.rand(2,2))
        cases.append( (A,1) )
        cases.append( (A,2) )
        A = csr_matrix(scipy.zeros((3,3)))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = csr_matrix(scipy.rand(3,3))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = poisson( (4,4), format='csr')
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,4) )
        A = array([[ 9.1,  9.8,  9.6,  0. ,  3.6,  0. ],
                   [18.2, 19.6,  0. ,  0. ,  1.7,  2.8],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  4.2,  1. ,  1.1],
                   [ 0. ,  0. ,  9.1,  0. ,  0. ,  9.3]])
        A = csr_matrix(A)
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,3) )

        # reference implementation of 1 iteration
        def gold(A, x, b, blocksize, sweep):

            A = csr_matrix(A)
            Dinv = get_block_diag(A, blocksize=blocksize, inv_flag=True)
            D = get_block_diag(A, blocksize=blocksize, inv_flag=False)
            A_no_D = A - bsr_matrix( (D, scipy.arange(Dinv.shape[0]), scipy.arange(Dinv.shape[0] + 1)), shape=A.shape)
            A_no_D = csr_matrix(A_no_D)

            if sweep == 'symmetric':
                x = gold(A, x, b, blocksize, 'forward')
                x = gold(A, x, b, blocksize, 'backward')
                return x
            elif sweep == 'forward':
                start,stop,step = (0, A.shape[0], blocksize)
            elif sweep == 'backward':
                start,stop,step = (A.shape[0] - blocksize, -blocksize, -blocksize)

            for i in range(start, stop, step):
                r = A_no_D[i:(i+blocksize),:]*x
                r = scipy.mat(Dinv[i/blocksize,:,:])*scipy.mat(scipy.ravel(b[i:(i+blocksize)]) - scipy.ravel(r)).reshape(-1,1)
                x[i:(i+blocksize)] =  scipy.ravel(r)

            return x


        for A,blocksize in cases:
            b = rand(A.shape[0])
            # forward
            x = rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='forward', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'forward'), decimal=4 )
            # backward
            x = rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='backward', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'backward'), decimal=4 )
            # symmetric
            x = rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='symmetric', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'symmetric'), decimal=4 )

        # check for aggreement between gauss_seidel and block gauss-seidel with blocksize=1
        A = poisson( (4,5), format='csr')
        b = rand(A.shape[0])
        # forward
        x = rand(A.shape[0])
        x_copy = x.copy()
        block_gauss_seidel(A, x, b, iterations=2, sweep='forward', blocksize=1)
        gauss_seidel(A, x_copy, b, iterations=2, sweep='forward')
        assert_almost_equal( x, x_copy, decimal=4 )
        # backward
        x = rand(A.shape[0])
        x_copy = x.copy()
        block_gauss_seidel(A, x, b, iterations=2, sweep='backward', blocksize=1)
        gauss_seidel(A, x_copy, b, iterations=2, sweep='backward')
        assert_almost_equal( x, x_copy, decimal=4 )
        # symmetric
        x = rand(A.shape[0])
        x_copy = x.copy()
        block_gauss_seidel(A, x, b, iterations=2, sweep='symmetric', blocksize=1)
        gauss_seidel(A, x_copy, b, iterations=2, sweep='symmetric')
        assert_almost_equal( x, x_copy, decimal=4 )


        # complex valued tests
        cases = []
        A = csr_matrix(scipy.rand(3,3) + 1.0j*scipy.rand(3,3))
        cases.append( (A,1) )
        cases.append( (A,3) )
        A = poisson( (4,4), format='csr')
        A.data = A.data + 1.0j*scipy.rand(A.data.shape[0])
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,4) )
        A = array([[ 9.1j,  9.8j,  9.6,  0. ,  3.6,  0. ],
                   [18.2j, 19.6j,  0. ,  0. ,  1.7,  2.8],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  0. ,  0. ,  0. ],
                   [ 0. ,  0. ,  0. ,  4.2, 1.0j,  1.1],
                   [ 0. ,  0. ,  9.1,  0. ,  0. ,  9.3]])
        A = csr_matrix(A)
        cases.append( (A,1) )
        cases.append( (A,2) )
        cases.append( (A,3) )

        for A,blocksize in cases:
            b = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            # forward
            x = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='forward', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'forward'), decimal=4 )
            # backward
            x = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='backward', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'backward'), decimal=4 )
            # symmetric
            x = rand(A.shape[0]) + 1.0j*rand(A.shape[0])
            x_copy = x.copy()
            block_gauss_seidel(A, x, b, iterations=1, sweep='symmetric', blocksize=blocksize)
            assert_almost_equal( x, gold(A, x_copy, b, blocksize, 'symmetric'), decimal=4 )


#class TestDispatch(TestCase):
#    def test_string(self):
#        from pyamg.relaxation import dispatch
#
#        A = poisson( (4,), format='csr')
#
#        cases = []
#        cases.append( 'gauss_seidel' )
#        cases.append( ('gauss_seidel',{'iterations':3}) )
#
#        for case in cases:
#            fn = dispatch(case)
#            fn(A, ones(4), zeros(4))





import numpy
from numpy import inner, conjugate, asarray, mod, ravel, sqrt
from scipy.sparse.linalg.isolve.utils import make_system
from scipy.sparse.sputils import upcast
from pyamg.util.linalg import norm
from warnings import warn

__docformat__ = "restructuredtext en"

__all__ = ['steepest_descent']

def steepest_descent(A, b, x0=None, tol=1e-5, maxiter=None, xtype=None, M=None, 
       callback=None, residuals=None):
    '''Steepest descent algorithm
    
    Solves the linear system Ax = b. Left preconditioning is supported.

    Parameters
    ----------
    A : {array, matrix, sparse matrix, LinearOperator}
        n x n, linear system to solve
    b : {array, matrix}
        right hand side, shape is (n,) or (n,1)
    x0 : {array, matrix}
        initial guess, default is a vector of zeros
    tol : float
        relative convergence tolerance, i.e. tol is scaled by the
        preconditioner norm of r_0, or ||r_0||_M.
    maxiter : int
        maximum number of allowed iterations
    xtype : type
        dtype for the solution, default is automatic type detection
    M : {array, matrix, sparse matrix, LinearOperator}
        n x n, inverted preconditioner, i.e. solve M A x = M b.
    callback : function
        User-supplied function is called after each iteration as
        callback(xk), where xk is the current solution vector
    residuals : list
        residuals contains the residual norm history,
        including the initial residual.  The preconditioner norm
        is used, instead of the Euclidean norm.
     
    Returns
    -------    
    (xNew, info)
    xNew : an updated guess to the solution of Ax = b
    info : halting status of cg

            ==  ======================================= 
            0   successful exit
            >0  convergence to tolerance not achieved,
                return iteration count instead.  
            <0  numerical breakdown, or illegal input
            ==  ======================================= 

    Notes
    -----
    The LinearOperator class is in scipy.sparse.linalg.interface.
    Use this class if you prefer to define A or M as a mat-vec routine
    as opposed to explicitly constructing the matrix.  A.psolve(..) is
    still supported as a legacy.

    The residual in the preconditioner norm is both used for halting and
    returned in the residuals list. 

    Examples
    --------
    >>> from pyamg.krylov import steepest_descent
    >>> from pyamg.util.linalg import norm
    >>> import numpy 
    >>> from pyamg.gallery import poisson
    >>> A = poisson((10,10))
    >>> b = numpy.ones((A.shape[0],))
    >>> (x,flag) = steepest_descent(A,b, maxiter=2, tol=1e-8)
    >>> print norm(b - A*x)
    7.89436429704

    References
    ----------
    .. [1] Yousef Saad, "Iterative Methods for Sparse Linear Systems, 
       Second Edition", SIAM, pp. 137--142, 2003
       http://www-users.cs.umn.edu/~saad/books.html

    '''
    A,M,x,b,postprocess = make_system(A,M,x0,b,xtype=None)
    n = len(b)
    
    ##
    # Ensure that warnings are always reissued from this function
    import warnings
    warnings.filterwarnings('always', module='pyamg\.krylov\._steepest_descent')

    # determine maxiter
    if maxiter is None:
        maxiter = int(len(b)) 
    elif maxiter < 1:
        raise ValueError('Number of iterations must be positive')
    
    # setup method
    r  = b - A*x
    z  = M*r
    rz = inner(r.conjugate(), z)

    # use preconditioner norm
    normr = sqrt(rz)

    if residuals is not None:
        residuals[:] = [normr] #initial residual 

    # Check initial guess ( scaling by b, if b != 0, 
    #   must account for case when norm(b) is very small)
    normb = norm(b)
    if normb == 0.0:
        normb = 1.0
    if normr < tol*normb:
        return (postprocess(x), 0)

    # Scale tol by ||r_0||_M
    if normr != 0.0:
        tol = tol*normr
   
    # How often should r be recomputed
    recompute_r = 50

    iter = 0

    while True:
        iter = iter+1

        q = A*z
        zAz = inner(z.conjugate(), q)                # check curvature of A
        if zAz < 0.0:
            warn("\nIndefinite matrix detected in steepest descent, aborting\n")
            return (postprocess(x), -1)


        alpha = rz / zAz                            # step size
        x = x + alpha*z
        
        if mod(iter, recompute_r) and iter > 0:
            r = b - A*x
        else:
            r = r - alpha*q
        
        z = M*r
        rz = inner(r.conjugate(), z)
        
        if rz < 0.0:                                # check curvature of M
            warn("\nIndefinite preconditioner detected in steepest descent, aborting\n")
            return (postprocess(x), -1)

        normr = sqrt(rz)                            # use preconditioner norm

        if residuals is not None:
            residuals.append(normr)
        
        if callback is not None:
            callback(x)

        if normr < tol:
            return (postprocess(x), 0)
        elif rz == 0.0:
            # important to test after testing normr < tol. rz == 0.0 is an
            # indicator of convergence when r = 0.0
            warn("\nSingular preconditioner detected in steepest descent, ceasing iterations\n")
            return (postprocess(x), -1)
        
        if iter == maxiter:
            return (postprocess(x), iter)

#if __name__ == '__main__':
#    # from numpy import diag
#    # A = random((4,4))
#    # A = A*A.transpose() + diag([10,10,10,10])
#    # b = random((4,1))
#    # x0 = random((4,1))
#
#    from pyamg.gallery import stencil_grid
#    from pyamg import smoothed_aggregation_solver
#    from numpy.random import random
#    from numpy import ravel, inner
#    A = stencil_grid([[0,-1,0],[-1,4,-1],[0,-1,0]],(100,100),dtype=float,format='csr')
#    b = random((A.shape[0],))
#    x0 = random((A.shape[0],))
#    
#    fvals = []
#    def callback(x):
#        x = ravel(x)
#        fvals.append( 0.5*inner(x, A*x) - inner(ravel(b),x) )
#
#    print '\n\nTesting steepest descent with %d x %d 2D Laplace Matrix'%(A.shape[0],A.shape[0])
#    resvec = []
#    sa = smoothed_aggregation_solver(A)
#    (x,flag) = steepest_descent(A,b,x0,tol=1e-8,maxiter=20,residuals=resvec, M=sa.aspreconditioner(), callback=callback)
#    print 'Funcation values:  ' + str(fvals)
#    print 'initial norm = %g'%(norm(b - A*x0))
#    print 'final norm = %g'%(norm(b - A*x))
#    print 'info flag = %d'%(flag)
#

